# Zope object that allows storage of simple constants.
# Can be useful in the DatabaseAPI.
# Copyright (C) 2000 Shane Hathaway
# E-mail: shathaway@earthling.net or hathawsh@pi.slcc.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__version__ = '0.1'

from Globals import Persistent, HTMLFile, MessageDialog, PersistentMapping
import string
import OFS.SimpleItem
import Acquisition
import AccessControl
from AccessControl.PermissionRole import PermissionRole


def _debug(msg): print msg
#def _debug(msg): pass

def manage_addConstant(self, id, title='', type='string', k='', REQUEST=None):
    '''Adds a Constant to a folder.'''
    self._setObject(id, Constant(id, title, type, k))
    return self.manage_main(self, REQUEST)

manage_addConstantForm = HTMLFile('addConstant', globals())

def getConstantTypes():
    '''Gets the list of available types of constants.
    '''
    return Constant.kTypesList


class KType:
    def __init__(self, name, desc):
        self.name = name
        self.desc = desc


class Constant (Persistent,
                OFS.SimpleItem.Item,
                Acquisition.Implicit,
                AccessControl.Role.RoleManager):
    '''A simple object that holds a constant.
    '''

    meta_type = 'Constant'

    manage_options=(
        {'label':'Value',              'action':'manage_main'},
        {'label':'Security',           'action':'manage_access'},
        {'label':'Undo',               'action':'manage_UndoForm'},
    )

    __ac_permissions__=(
        ('View management screens', ('manage_constantEdit',
                                     'manage_main',)),
        )

    manage_main = HTMLFile('editConstant', globals())

    kTypesList = [
        KType('string', 'String'),
        KType('init', 'Integer'),
        KType('float', 'Floating-point')
        ]

    def __init__(self, id, title, type, k):
        self.id = id
        self.title = title
        self.type = type
        self.__setK(k)

    def __setK(self, val):
        '''Sets the value of the constant, relying on existing
        mechanisms for validity checking.
        '''
        val = string.strip(val)
        if self.type == 'float':
            val = string.atof(val)
        elif self.type == 'int':
            val = string.atol(val)
        self.k = None  # Clear it first.
        self.k = val

    def getConstantTypes(self):
        '''Gets the list of available types of constants.
        '''
        return self.kTypesList
    
    def manage_constantEdit(self, REQUEST):
        '''Edits the value of the constant and/or title.'''
        if REQUEST.has_key('title'):
            self.title = REQUEST['title']
        if REQUEST.has_key('type'):
            self.type = REQUEST['type']
        if REQUEST.has_key('k'):
            self.__setK(REQUEST['k'])
        return MessageDialog(
            title  ='Changes accepted.',
            message='Changes accepted.',
            action ='manage_main')

    def __call__(self, *args, **kw):
        '''Returns the value of the stored constant.
        '''
        return self.k
